@@ -35,7 +35,7 @@
  */
 
 #include "includes.h"
-RCSID("$OpenBSD: sshtty.c,v 1.3 2002/03/04 17:27:39 stevesk Exp $");
+RCSID("$OpenBSD: sshtty.c,v 1.5 2003/09/19 17:43:35 markus Exp $");
 
 #include "sshtty.h"
 #include "log.h"
@@ -43,12 +43,6 @@ RCSID("$OpenBSD: sshtty.c,v 1.3 2002/03/04 17:27:39 stevesk Exp $");
 static struct termios _saved_tio;
 static int _in_raw_mode = 0;
 
-int
-in_raw_mode(void)
-{
-	return _in_raw_mode;
-}
-
 struct termios
 get_saved_tio(void)
 {
@@ -64,8 +58,6 @@ leave_raw_mode(void)
 		perror("tcsetattr");
 	else
 		_in_raw_mode = 0;
-
-	fatal_remove_cleanup((void (*) (void *)) leave_raw_mode, NULL);
 }
 
 void
@@ -80,6 +72,9 @@ enter_raw_mode(void)
 	_saved_tio = tio;
 	tio.c_iflag |= IGNPAR;
 	tio.c_iflag &= ~(ISTRIP | INLCR | IGNCR | ICRNL | IXON | IXANY | IXOFF);
+#ifdef IUCLC
+	tio.c_iflag &= ~IUCLC;
+#endif
 	tio.c_lflag &= ~(ISIG | ICANON | ECHO | ECHOE | ECHOK | ECHONL);
 #ifdef IEXTEN
 	tio.c_lflag &= ~IEXTEN;
@@ -91,6 +86,4 @@ enter_raw_mode(void)
 		perror("tcsetattr");
 	else
 		_in_raw_mode = 1;
-
-	fatal_add_cleanup((void (*) (void *)) leave_raw_mode, NULL);
 }
